# Documentation

This folder contains examples and documentation for the Deep Java Library (DJL) project.

## Modules

- [Core API](../api/README.md) - Engine-agnostic API
- [Basic Dataset](../basicdataset/README.md) - Built-in datasets
- [Model Zoo](../model-zoo/README.md) - Built-in engine-agnostic model zoo
- [Repository API](../repository/README.md) - Repository utilities to build the model zoo and dataset
- [MXNet Engine](../mxnet/mxnet-engine/README.md) - MXNet Engine implementation
- [MXNet Model Zoo](../mxnet/mxnet-model-zoo/README.md) - MXNet symbolic model zoo

## Jupyter notebook tutorials

- [Create your first neural network](../jupyter/create_your_first_network.ipynb)
- [Train your first model](../jupyter/train_your_first_model.ipynb)
- [Run image classification with your first model](../jupyter/image_classification_with_your_model.ipynb)
- [Run object detection with model zoo](../jupyter/object_detection_with_model_zoo.ipynb)
- [Load pre-trained MXNet model](../jupyter/load_mxnet_model.ipynb)
- [Transfer learning example](../jupyter/transfer_learning_on_cifar10.ipynb)
- [Question answering example](../jupyter/BERTQA.ipynb)

## [API Examples](../examples/README.md)

- [Single-shot Object Detection example](../examples/docs/object_detection.md)
- [Train your first model](../examples/docs/train_mnist_mlp.md)
- [Image classification example](../examples/docs/image_classification.md)
- [Transfer learning example](../examples/docs/train_cifar10_resnet.md)
- [Train SSD model example](../examples/docs/train_pikachu_ssd.md)
- [Multi-threaded inference example](../examples/docs/multithread_inference.md)
- [Bert question and answer example](../examples/docs/BERT_question_and_answer.md)
- [Instance segmentation example](../examples/docs/instance_segmentation.md)
- [Pose estimation example](../examples/docs/pose_estimation.md)
- [Action recognition example](../examples/docs/action_recognition.md)

## Cheat sheet

- [How to collect metrics](how_to_collect_metrics.md)
- [How to use a dataset](development/how_to_use_dataset.md)

## [Contributor Documentation](development/README.md)

## [FAQ](faq.md)
